﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace FeatureGuideCharp
{
    class _10_24
    {
        /*
    * Return the next MIN based on the organizational ID and the next sequence
    * number.
    */
        public String minFromSeqNum(long aOrgId, long seqNum)
        {
            byte[] m = new byte[17];
            longToDigits(seqNum, m, 0, 10);
            longToDigits(aOrgId, m, 10, 7);
            StringBuilder res = new StringBuilder(m.Length + 1);
            for (int i = m.Length - 1; i >= 0; --i)
            {
                res.Append(m[i].ToString().Single());
            }
            res.Append(checkDigit(m).ToString().Single());
            return res.ToString();
        }

        /*
         * Fill part of an array with the decimal digits of a given value. The
         * values are stored in increasing order of significance. Trailing zeroes
         * will be appended if the array is larger than necessary, while the
         * sequence of digits will be truncated if the space is smaller than
         * necessary.
         */
        private void longToDigits(long val, byte[] b, int off, int len)
        {
            for (int i = off, d = 1; i < off + len; i++)
            {
                b[i] = (byte)((val / d) % 10);
                d *= 10;
            }
        }

        /*
         * Calculate the check digit.
         *
         * The algorithm is described on page 10 of the "MERS eRegistry Integration
         * Handbook: Volume II of IV" release 1.1, dated October 6, 2004.
         */
        private byte checkDigit(byte[] m)
        {
            int sum = 0;
            int carry = 0;
            for (int i = 0; i < m.Length; i += 2)
            {
                int p = m[i] * 2;
                sum += carry + p % 10;
                carry = p / 10;
            }
            sum += carry;
            for (int i = 1; i < m.Length; i += 2)
            {
                sum += m[i];
            }
            int tot = (sum + 9) / 10;
            tot *= 10;
            return (byte)(tot - sum);
        }


        public void execute()
        {
            String minFromSeqNum = new _10_24().minFromSeqNum(9999994, 1000010059);
            Debugger.Log(1, "Debug", minFromSeqNum + "\n");         //expected result is 999999410000100591


        }



    }
}
